"""
This script enumerates all loaded modules, creates a python23.zip for each
and copyies associated .pyds. The aim is to prepare a package of modules 
you will need to distribute.

Usage: When your program ends, add these two lines:

import dumpmods
dumpmods.analyze_modules(<path to store files in>)

where <path to store files in> has obvious connotations.
"""
import sys
import os
import shutil
import zipfile
import traceback
try:
    import cfrompy
except ImportError:
    print "*** WARNING, cfrompy not available ***"
    cfrompy = None

def getarcname(filename):
    for p in sys.path:
        p = p.lower()
        if filename.lower().startswith(p):
            return filename[len(p)+1:]
    return filename

def analyze_modules(scriptname, targetdir):    
    print "Analyzing modules needed..."
    
    try:
        exec('import %s' % scriptname)
    except:
        print "ERROR, unable to import '%s'" % scriptname
        traceback.print_exc()
        return
    
    try:
        os.makedirs(targetdir)
    except:
        pass
    
    zname = os.path.join(targetdir, "python23.zip")
    print "Creating %s..." % zname
    z = zipfile.ZipFile(zname, "w", zipfile.ZIP_DEFLATED)
    
    encpath = None
    
    for modulename in sys.modules.keys():
        module = sys.modules[modulename]
        try:
            filename = module.__file__
        except:
            if repr(module).find("(built-in)") < 0:
                print "---> IGNORING %s (%s)" % (modulename, module)
            continue

        # windows:
        oldname = filename[:]
        filename = filename.lower()

        if filename.endswith(".pyc") or filename.endswith(".py"):
            arcname = getarcname(oldname)
            if arcname.startswith("encodings\\"):
                print "skipping encodings, will be filled in later..."
                if not encpath:
                    encpath = os.path.dirname(filename)
            else:
                print "adding %s as %s" % (filename, arcname)
                z.write(filename, arcname)
    
        elif filename.endswith(".pyd"):
            nname = os.path.join(targetdir, os.path.basename(filename))
            print "copying %s as %s" % (filename, nname)
            shutil.copy( filename, nname )
            
    # add encodings
    if encpath:
        for filename in os.listdir(encpath):
            source = os.path.join(encpath, filename)
            target = "encodings\\%s" % filename
            print "adding %s as %s" % (source, target)
            z.write(source, target)

    # add binary stuff
    if cfrompy:
        for module in ('', 'python23.dll', 'python23_d.dll', 'PyWinTypes23.dll',  'PyWinTypes23_d.dll', 'msvcrtd.dll' ):
            modulename = cfrompy.GetModuleFilename(module)
            if modulename:
                nname = os.path.join(targetdir, os.path.basename(modulename))
                print "copying %s as %s" % (modulename, nname)
                shutil.copy( modulename, nname )
            else:
                print "%s currently not loaded..." % module

    z.close()
    
if __name__ == "__main__":
    analyze_modules("test", "testworld")
    