#include "windows.h"
#include "python.h"

PyThreadState* g_pyThreadState;
PyObject* g_pyMain;

int g_argc;
char** g_argv;

PyObject* Py_ReturnNone()
{
    PyObject* result = Py_None;
    Py_INCREF(result);
    return result;
}

PyObject* func_GetSysArgv(PyObject* self, PyObject* args)
{
    PyObject* result = PyList_New( g_argc - 1 );
    if( result )
    {
        char buffer[1024];
        int i;

        if( !GetModuleFileName(0, buffer, sizeof(buffer) ) )
            buffer[0] = 0;

        PyList_SET_ITEM( result, 0, PyString_FromString( buffer ) );
        for( i = 0; i < g_argc - 2; i++ )
        {
            PyList_SET_ITEM( result, i+1, PyString_FromString( g_argv[i+2] ) );
        }
    }
    return result;
}

PyObject* func_GetModuleFilename(PyObject* self, PyObject* args)
{
    char* arg;
    HMODULE hModule;
    char buffer[1024];
    

    if( !PyArg_ParseTuple(args,"s",&arg) )
        return NULL;

    if( !*arg )
    {
        if( !GetModuleFileName(0, buffer, sizeof(buffer) ) )
            buffer[0] = 0;
    }
    else
    {
        hModule = GetModuleHandle(arg);
        if( !hModule )
        {
            buffer[0] = 0;
        }
        else if( !GetModuleFileName(hModule, buffer, sizeof(buffer) ) )
        {
            buffer[0] = 0;
        }
    }
    return PyString_FromString(buffer);
}

PyMethodDef cfrompy_methods[] =
{   
    { "GetModuleFilename", func_GetModuleFilename, METH_VARARGS,NULL},
    { "GetSysArgv", func_GetSysArgv, METH_VARARGS,NULL},
    { NULL, NULL, 0, 0 }
};

        
void Py_Dumpmods(char* script, char* archive)
{
    PyObject* m_pyFunc;
    PyObject* m_pyArgs;
    PyGILState_STATE m_GIL;
    PyObject* pyValue;

    m_GIL = PyGILState_Ensure();
    m_pyFunc = PyDict_GetItemString(PyModule_GetDict(g_pyMain), "analyze_modules");

    if( m_pyFunc && PyCallable_Check(m_pyFunc))
    {
        m_pyArgs = PyTuple_New(2);
        if( m_pyArgs )
        {
            PyTuple_SetItem(m_pyArgs, 0, PyString_FromString(script));
            PyTuple_SetItem(m_pyArgs, 1, PyString_FromString(archive));

            pyValue = PyObject_CallObject(m_pyFunc, m_pyArgs);
            if( pyValue )
            {
                Py_DECREF(pyValue);
                pyValue = NULL;
            }
            Py_DECREF(m_pyArgs);
            m_pyArgs = NULL;
        }
    }
    else 
    {
        m_pyFunc = NULL;
    }
    PyGILState_Release(m_GIL);
}

        
void Pyxe_Main()
{
    PyObject* m_pyFunc;
    PyObject* m_pyArgs;
    PyGILState_STATE m_GIL;

    m_GIL = PyGILState_Ensure();
    m_pyFunc = PyDict_GetItemString(PyModule_GetDict(g_pyMain), "main");

    if( m_pyFunc && PyCallable_Check(m_pyFunc))
    {
        m_pyArgs = PyTuple_New(0);
        if( m_pyArgs )
        {
            PyObject* pyValue = PyObject_CallObject(m_pyFunc, m_pyArgs);
            if( pyValue )
            {
                Py_DECREF(pyValue);
                pyValue = NULL;
            }
            Py_DECREF(m_pyArgs);
            m_pyArgs = NULL;
        }
    }
    else 
    {
        m_pyFunc = NULL;
    }
    PyGILState_Release(m_GIL);
}

BOOL Py_Startup(char* script)
{
    PyObject* pName;
    char buffer[1024] = "", format[256];
    

    GetModuleFileName(0, buffer, sizeof(buffer) );
    sprintf( format, "PYTHONPATH=%s", buffer );
    putenv(buffer);
    
    Py_Initialize();
    PyEval_InitThreads();

    Py_InitModule4("cfrompy",cfrompy_methods,"cfrompy",(PyObject *)NULL,PYTHON_API_VERSION); 

    pName = PyString_FromString(script);
    g_pyMain = PyImport_Import(pName);
    Py_DECREF(pName);
    
    g_pyThreadState = PyThreadState_Get();
    PyEval_ReleaseThread(g_pyThreadState);
    return (g_pyMain != NULL);
}

int main( int argc, char** argv )
{
    if( (argc == 3) && (!stricmp(argv[1], "/BUILD")) )
    {
        if( !Py_Startup("pyxe_dumpmods") )
        {
            printf("ERROR, unable to import pyxe_dumpmods\n");
        }
        else
        {
            Py_Dumpmods(argv[2], argv[2]);
        }
    }
    else if( argc >= 2 )
    {
        g_argc = argc;
        g_argv = argv;

        if( !Py_Startup(argv[1]) )
        {
            printf("ERROR, unable to import %s\n", argv[1]);
        }
        else
        {
            Pyxe_Main();
        }
    }
    else
    {
        printf("USAGE 1: pyxe <script>\n"
               "USAGE 2: pyxe /BUILD <script>\n");
    }
    __try
    {
        if( g_pyMain )
        {
            Py_DECREF(g_pyMain);
            g_pyMain = NULL;
        }
        if( g_pyThreadState )
        {
            PyEval_AcquireThread(g_pyThreadState);
            g_pyThreadState = NULL;
        }
        Py_Finalize();
    }
    __except( EXCEPTION_EXECUTE_HANDLER )
    {
    }
    return 0;
} // main()



